"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __exportStar = (this && this.__exportStar) || function(m, exports) {
    for (var p in m) if (p !== "default" && !Object.prototype.hasOwnProperty.call(exports, p)) __createBinding(exports, m, p);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.middlewareCallback = exports.middlewareAccessApi = exports.middlewareAuth = exports.isAccessApi = void 0;
const helpers_1 = require("../helpers");
const roleHasPermission_model_1 = require("./RoleAndPermissions/roleHasPermission.model");
const errorBuilder_1 = require("../errorBuilder");
const injector_1 = require("../di/injector");
const consts_1 = require("./consts");
const authentication_1 = require("../authentication");
const isAccessApi = (indentity, api) => __awaiter(void 0, void 0, void 0, function* () {
    const permisions = yield (0, helpers_1.getQueryModel)(roleHasPermission_model_1.RoleHasPermission)
        .with({ model: "permission" })
        .find({ roleId: { $eq: indentity.roleId } });
    return !!permisions.find((e) => {
        if (!e.permission)
            return false;
        return e.permission.method.toLocaleUpperCase() === api.method.toLocaleUpperCase() && e.permission.api == api.path;
    });
});
exports.isAccessApi = isAccessApi;
const middlewareAuth = (req, res, next, api) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const authHeader = req.header("authorization");
        if (!authHeader || !authHeader.startsWith("Bearer ")) {
            throw errorBuilder_1.ErrorBuilder.unauthorized();
        }
        const token = authHeader.substring(7, authHeader.length);
        const jwtService = injector_1.injector.get(authentication_1.JwtService);
        let data;
        try {
            data = jwtService.verify(token);
        }
        catch (error) {
            throw errorBuilder_1.ErrorBuilder.unauthorized();
        }
        req.user = data;
        req.token = token;
        return next();
    }
    catch (error) {
        res._error = error;
        const statusCode = error instanceof errorBuilder_1.ErrorBuilder ? error.statusCode : 401;
        return res.status(statusCode).broadcast({
            status: false,
            name: error.name,
            message: error.message,
        });
    }
});
exports.middlewareAuth = middlewareAuth;
const middlewareAccessApi = (req, res, next, api) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        if (!req.user) {
            throw errorBuilder_1.ErrorBuilder.unauthorized();
        }
        if (req.user.roleName == consts_1.RoleAdmin) {
            return next();
        }
        const isAccess = yield (0, exports.isAccessApi)(req.user, api);
        if (!isAccess) {
            throw errorBuilder_1.ErrorBuilder.accessDenied("Access denied");
        }
        return next();
    }
    catch (error) {
        res._error = error;
        const statusCode = error instanceof errorBuilder_1.ErrorBuilder ? error.statusCode : 403;
        return res.status(statusCode).broadcast({
            status: false,
            name: error.name,
            message: error.message,
        });
    }
});
exports.middlewareAccessApi = middlewareAccessApi;
const middlewareCallback = (req, res, next, callback) => __awaiter(void 0, void 0, void 0, function* () {
    const authHeader = req.header("authorization");
    try {
        const passCallback = yield callback(req, res, next);
        if (!passCallback) {
            throw errorBuilder_1.ErrorBuilder.accessDenied();
        }
        return next();
    }
    catch (error) {
        res._error = error;
        const statusCode = error instanceof errorBuilder_1.ErrorBuilder ? error.statusCode : 403;
        return res.status(statusCode).broadcast({
            status: false,
            name: error.name,
            message: error.message,
        });
    }
});
exports.middlewareCallback = middlewareCallback;
__exportStar(require("./RoleAndPermissions/role.model"), exports);
__exportStar(require("./RoleAndPermissions/permission.model"), exports);
__exportStar(require("./RoleAndPermissions/roleHasPermission.model"), exports);
