import { DataTypes, QueryInterface, Transaction } from "sequelize";
export interface MigrationModule {
    up: (queryInterface: QueryInterface, t: Transaction) => Promise<void>;
    down: (queryInterface: QueryInterface, t: Transaction) => Promise<void>;
}
export declare const MIGRATION_TEMPLATE = "\n    import { DataTypes, QueryInterface, Transaction } from \"@fasterjs/core\";\n\n    export default {\n        up: async (queryInterface: QueryInterface, t: Transaction) => {\n            /** no-op */\n        },\n        down: async (queryInterface: QueryInterface, t: Transaction) => {\n            /** no-op */\n        },\n    };\n";
export declare const MIGRATION_ROLE_PERMISSION_TEMPLATE = "\n    import { DataTypes, QueryInterface, Transaction, Configuration, ServiceProvider, Config } from \"@fasterjs/core\";\n\n    export default {\n        up: async (queryInterface: QueryInterface, t: Transaction) => {\n            /** no-op */\n            const config = ServiceProvider.getInstance(Configuration) as Config;\n            const tableRole = config.ROLE_AND_PERMISSION?.TABLE_ROLE || \"Roles\"\n            const tablePermission = config.ROLE_AND_PERMISSION?.TABLE_PERMISSION || \"Permissions\"\n            const tableRolePermission = config.ROLE_AND_PERMISSION?.TABLE_ROLE_PERMISSION || \"RoleHasPermissions\"\n\n            await queryInterface.createTable(tableRole, {\n                id: { primaryKey: true, allowNull: false, type: DataTypes.STRING(50) },\n                name: { allowNull: false, type: DataTypes.STRING(200) },\n                createdAt: {\n                    allowNull: false, type: DataTypes.BIGINT({\n                        unsigned: true,\n                    })\n                },\n                updatedAt: {\n                    allowNull: false, type: DataTypes.BIGINT({\n                        unsigned: true,\n                    })\n                }\n            }, {\n                transaction: t\n            })\n\n            await queryInterface.createTable(tablePermission, {\n                id: { primaryKey: true, allowNull: false, type: DataTypes.STRING(50) },\n                name: { allowNull: false, type: DataTypes.STRING(200) },\n                method: { allowNull: false, type: DataTypes.STRING(10) },\n                api: { allowNull: false, type: DataTypes.STRING(300) },\n                createdAt: {\n                    allowNull: false, type: DataTypes.BIGINT({\n                        unsigned: true,\n                    })\n                },\n                updatedAt: {\n                    allowNull: false, type: DataTypes.BIGINT({\n                        unsigned: true,\n                    })\n                }\n            }, {\n                transaction: t\n            })\n\n            await queryInterface.createTable(tableRolePermission, {\n                id: { primaryKey: true, allowNull: false, type: DataTypes.STRING(50) },\n                roleId: { allowNull: false, type: DataTypes.STRING(50) },\n                permissionId: { allowNull: false, type: DataTypes.STRING(50) },\n                createdAt: {\n                    allowNull: false, type: DataTypes.BIGINT({\n                        unsigned: true,\n                    })\n                },\n                updatedAt: {\n                    allowNull: false, type: DataTypes.BIGINT({\n                        unsigned: true,\n                    })\n                }\n            }, {\n                transaction: t\n            })\n\n            await queryInterface.addConstraint(tableRolePermission, {\n                fields: [\"roleId\"],\n                type: \"foreign key\",\n                name: tableRolePermission + \"_roleId_fk\",\n                references: {\n                    table: tableRole,\n                    field: \"id\",\n                },\n                onDelete: \"CASCADE\",\n                onUpdate: \"CASCADE\",\n                transaction: t\n            })\n\n            await queryInterface.addConstraint(tableRolePermission, {\n                fields: [\"permissionId\"],\n                type: \"foreign key\",\n                name: tableRolePermission + \"_permissionId_fk\",\n                references: {\n                    table: tablePermission,\n                    field: \"id\",\n                },\n                onDelete: \"CASCADE\",\n                onUpdate: \"CASCADE\",\n                transaction: t\n            })\n\n        },\n        down: async (queryInterface: QueryInterface, t: Transaction) => {\n            /** no-op */\n            const config = ServiceProvider.getInstance(Configuration) as Config;\n            const tableRole = config.ROLE_AND_PERMISSION?.TABLE_ROLE || \"Roles\"\n            const tablePermission = config.ROLE_AND_PERMISSION?.TABLE_PERMISSION || \"Permissions\"\n            const tableRolePermission = config.ROLE_AND_PERMISSION?.TABLE_ROLE_PERMISSION || \"RoleHasPermissions\"\n\n            await queryInterface.removeConstraint(tableRolePermission, tableRolePermission + \"_permissionId_fk\", { transaction: t })\n            await queryInterface.removeConstraint(tableRolePermission, tableRolePermission+ \"_roleId_fk\", { transaction: t })\n            await queryInterface.dropTable(tableRolePermission, { transaction: t })\n            await queryInterface.dropTable(tablePermission, { transaction: t })\n            await queryInterface.dropTable(tableRole, { transaction: t })\n        },\n    };\n";
export declare const metaTableDefinition: {
    name: {
        primaryKey: boolean;
        allowNull: boolean;
        type: DataTypes.StringDataType;
    };
};
export declare enum MigratorCommand {
    MAKE_ROLE_AND_PERMISSION = "makeRAP",
    GENERATE = "generate",
    UPGRADE = "up",
    DOWNGRADE = "down"
}
